/*
 * Jsonix is a JavaScript library which allows you to convert between XML
 * and JavaScript object structures.
 *
 * Copyright (C) 2010  Aleksei Valikov, Highsource.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

Jsonix.Context = Jsonix.Class({
	elementInfos : null,
	properties : null,
	substitutionMembersMap : null,
	scopedElementInfosMap : null,
	initialize : function(schemas, properties) {

		this.elementInfos = [];
		this.properties = {
			namespacePrefixes : {}
		};
		Jsonix.Util.Ensure.ensureArray(schemas);
		for ( var index = 0; index < schemas.length; index++) {
			var schema = schemas[index];
			Jsonix.Util.Ensure.ensureArray(schema.elementInfos);
			for ( var kndex = 0; kndex < schema.elementInfos.length; kndex++) {
				this.elementInfos.push(schema.elementInfos[kndex]);
			}
		}

		if (Jsonix.Util.Type.isObject(properties)) {
			if (Jsonix.Util.Type.isObject(properties.namespacePrefixes)) {
				this.properties.namespacePrefixes = properties.namespacePrefixes;
			}
		}

		this.substitutionMembersMap = {};
		this.scopedElementInfosMap = {};

		for ( var jndex = 0; jndex < this.elementInfos.length; jndex++) {
			var elementInfo = this.elementInfos[jndex];

			if (Jsonix.Util.Type.exists(elementInfo.substitutionHead)) {
				var substitutionHead = Jsonix.XML.QName.fromObject(elementInfo.substitutionHead);
				var substitutionHeadKey = substitutionHead.key;

				var substitutionMembers = this.substitutionMembersMap[substitutionHeadKey];

				if (!Jsonix.Util.Type.isArray(substitutionMembers)) {
					substitutionMembers = [];
					this.substitutionMembersMap[substitutionHeadKey] = substitutionMembers;
				}
				substitutionMembers.push(elementInfo);
			}

			var scopeKey;
			if (Jsonix.Util.Type.exists(elementInfo.scope)) {
				scopeKey = elementInfo.scope.name;
			} else {
				scopeKey = '##global';
			}

			var scopedElementInfos = this.scopedElementInfosMap[scopeKey];

			if (!Jsonix.Util.Type.isObject(scopedElementInfos)) {
				scopedElementInfos = {};
				this.scopedElementInfosMap[scopeKey] = scopedElementInfos;
			}
			scopedElementInfos[elementInfo.elementName.key] = elementInfo;
		}
	},
	getElementInfo : function(name, scope) {
		if (Jsonix.Util.Type.exists(scope)) {
			var scopeKey = scope.name;
			var scopedElementInfos = this.scopedElementInfosMap[scopeKey];
			if (Jsonix.Util.Type.exists(scopedElementInfos)) {
				var scopedElementInfo = scopedElementInfos[name.key];
				if (Jsonix.Util.Type.exists(scopedElementInfo)) {
					return scopedElementInfo;
				}
			}
		}

		var globalScopeKey = '##global';
		var globalScopedElementInfos = this.scopedElementInfosMap[globalScopeKey];
		if (Jsonix.Util.Type.exists(globalScopedElementInfos)) {
			var globalScopedElementInfo = globalScopedElementInfos[name.key];
			if (Jsonix.Util.Type.exists(globalScopedElementInfo)) {
				return globalScopedElementInfo;
			}
		}
		return null;
		//
		// throw "Element [" + name.key
		// + "] could not be found in the given context.";
	},
	getSubstitutionMembers : function(name) {
		return this.substitutionMembersMap[Jsonix.XML.QName.fromObject(name).key];
	},
	createMarshaller : function() {
		return new Jsonix.Context.Marshaller(this);
	},
	createUnmarshaller : function() {
		return new Jsonix.Context.Unmarshaller(this);
	},
	CLASS_NAME : 'Jsonix.Context'
});