/*
 * Jsonix is a JavaScript library which allows you to convert between XML
 * and JavaScript object structures.
 *
 * Copyright (C) 2010  Aleksei Valikov, Highsource.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

Jsonix.DOM = {
	createDocument : function() {

		if (document.implementation && document.implementation.createDocument) {
			return document.implementation.createDocument('', '', null);
		} else {
			var doc = new ActiveXObject('MSXML2.DOMDocument');
			return doc;
		}
	},
	serialize : function(node) {
		Jsonix.Util.Ensure.ensureExists(node);

		if (typeof XMLSerializer !== 'undefined') {
			return (new XMLSerializer()).serializeToString(node);
		} else if (Jsonix.Util.Type.exists(node.xml)) {
			return node.xml;
		} else {
			throw 'Could not serialize the node, neither XMLSerializer nor the [xml] property were found.';
		}
	},
	parse : function(text) {
		Jsonix.Util.Ensure.ensureExists(text);
		if (typeof DOMParser != 'undefined') {
			return (new DOMParser()).parseFromString(text, 'application/xml');
		} else if (typeof ActiveXObject != 'undefined') {
			var doc = Jsonix.DOM.createDocument('', '');
			doc.loadXML(text);
			return doc;
		} else {
			var url = 'data:text/xml;charset=utf-8,' + encodeURIComponent(text);
			var request = new XMLHttpRequest();
			request.open('GET', url, false);
			if (request.overrideMimeType) {
				request.overrideMimeType("text/xml");
			}
			request.send(null);
			return request.responseXML;
		}
	},
	load : function(url, callback, options) {

		var request = Jsonix.Request.INSTANCE;

		request.issue(url, function(transport) {
			var result;
			if (Jsonix.Util.Type.exists(transport.responseXML) && Jsonix.Util.Type.exists(transport.responseXML.documentElement)) {
				result = transport.responseXML;
			} else if (Jsonix.Util.Type.isString(transport.responseText)) {
				result = Jsonix.DOM.parse(transport.responseText);
			} else {
				throw 'Response does not have valid [responseXML] or [responseText].';
			}
			callback(result);

		}, function(transport) {
			throw 'Could not retrieve XML from URL [' + url + '].';

		}, options);
	}
};