/*
 * Jsonix is a JavaScript library which allows you to convert between XML
 * and JavaScript object structures.
 *
 * Copyright (C) 2010  Aleksei Valikov, Highsource.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

Jsonix.Model.ElementsPropertyInfo = Jsonix.Class(Jsonix.Model.AbstractElementsPropertyInfo, {
	elementTypeInfos : null,
	elementTypeInfosMap : null,
	initialize : function(options) {
		Jsonix.Util.Ensure.ensureObject(options);
		Jsonix.Model.AbstractElementsPropertyInfo.prototype.initialize.apply(this, [ options ]);
		// TODO Ensure correct arguments
		Jsonix.Util.Ensure.ensureArray(options.elementTypeInfos);
		this.elementTypeInfos = options.elementTypeInfos;
		this.elementTypeInfosMap = {};
		for ( var index = 0; index < this.elementTypeInfos.length; index++) {
			var elementTypeInfo = this.elementTypeInfos[index];
			Jsonix.Util.Ensure.ensureObject(elementTypeInfo);
			this.elementTypeInfosMap[elementTypeInfo.elementName.key] = elementTypeInfo.typeInfo;
		}
	},
	unmarshalElement : function(context, input) {
		// TODO make sure it's the right event type
		var elementName = input.getName();
		var key = elementName.key;
		var typeInfo = this.elementTypeInfosMap[key];
		if (Jsonix.Util.Type.exists(typeInfo)) {
			return this.unmarshalElementTypeInfo(context, input, typeInfo);
		}
		// TODO better exception
		throw "Element [" + key + "] is not known in this context";
	},
	marshalElement : function(context, value, output) {
		for ( var index = 0; index < this.elementTypeInfos.length; index++) {
			var elementTypeInfo = this.elementTypeInfos[index];
			var typeInfo = elementTypeInfo.typeInfo;
			if (typeInfo.isInstance(value)) {
				var elementName = elementTypeInfo.elementName;
				this.marshalElementTypeInfo(context, value, elementName, typeInfo, output);
				return;
			}
		}
		throw "Could not find an element with type info supporting the value [" + value + "].";
	},
	buildStructureElements : function(context, structure) {
		for ( var index = 0; index < this.elementTypeInfos.length; index++) {
			var elementTypeInfo = this.elementTypeInfos[index];
			structure.elements[elementTypeInfo.elementName.key] = this;
		}
	},
	CLASS_NAME : 'Jsonix.Model.ElementsPropertyInfo'
});
