/*
 * Jsonix is a JavaScript library which allows you to convert between XML
 * and JavaScript object structures.
 *
 * Copyright (C) 2010  Aleksei Valikov, Highsource.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

Jsonix.Schema.XSD.List = Jsonix.Class(Jsonix.Schema.XSD.AnySimpleType, {
	typeName : null,
	typeInfo : null,
	separator : ' ',
	trimmedSeparator : ' ',
	simpleType : true,
	initialize : function(typeInfo, typeName, separator) {
		Jsonix.Util.Ensure.ensureObject(typeInfo);
		// TODO Ensure correct argument
		this.typeInfo = typeInfo;

		if (Jsonix.Util.Type.exists(typeName)) {
			// TODO Ensure correct argument
			this.typeName = typeName;
		}

		if (Jsonix.Util.Type.isString(separator)) {
			// TODO Ensure correct argument
			this.separator = separator;
		} else {
			this.separator = ' ';
		}

		var trimmedSeparator = Jsonix.Util.StringUtils.trim(this.separator);
		if (trimmedSeparator.length === 0) {
			this.trimmedSeparator = ' ';
		} else {
			this.trimmedSeparator = trimmedSeparator;
		}
	},
	print : function(value) {
		if (!Jsonix.Util.Type.exists(value)) {
			return null;
		}
		// TODO Exception if not an array
		Jsonix.Util.Ensure.ensureArray(value);
		var result = '';
		for ( var index = 0; index < value.length; index++) {
			if (index > 0) {
				result = result + this.separator;
			}
			result = result + this.typeInfo.print(value[index]);
		}
		return result;
	},
	parse : function(text) {
		Jsonix.Util.Ensure.ensureString(text);
		var items = text.split(this.trimmedSeparator);
		var result = [];
		for ( var index = 0; index < items.length; index++) {
			result.push(this.typeInfo.parse(Jsonix.Util.StringUtils.trim(items[index])));
		}
		return result;
	},
	CLASS_NAME : 'Jsonix.Schema.XSD.List'
});
